package	jp.co.sds_corp.hibiya.monitor;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

public class GraphLegend extends JPanel
{
	private static final long serialVersionUID = 1L;
	ResourceBundle	res;
	java.util.List<GraphParam>	listParam;
	JScrollPane sp;
	LegendModel model;
	JTable	table;
	Vector<SelectionListener>	vecSel = new Vector<SelectionListener>();

	public GraphLegend() {
		super();
		createControls();
	}

	protected void createControls() {
		res = ResourceBundle.getBundle( "resource.monitor" );
		String var;
		var = res.getString( "graph.legend.title" );
		setBorder( new TitledBorder( var ));
		setLayout( new BorderLayout() );
		setPreferredSize( new Dimension( 400, 85 ));

		model = new LegendModel();
		table = new JTable( model );
		table.getSelectionModel().addListSelectionListener(
			new ListSelectionListener()
		{
			public void valueChanged( ListSelectionEvent ev ) {
				fireSelectionChanged( ev );
			}
		});
		table.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		table.setAutoResizeMode( JTable.AUTO_RESIZE_NEXT_COLUMN );
		TableColumn col;
		col = table.getColumnModel().getColumn( 0 );
		col.setMaxWidth( 20 );
		col.setCellRenderer( new LegendRenderer() );
		col = table.getColumnModel().getColumn( 1 );
		sp = new JScrollPane( table );
		add( sp, BorderLayout.CENTER );
	}

	public void setParam( java.util.List<GraphParam> list ) {
		this.listParam = list;
		model.fireTableDataChanged();
	}

	public void update() {
		model.fireTableDataChanged();
	}

	public void addSelectionListener( SelectionListener l ) {
		vecSel.add( l );
	}
	public void fireSelectionChanged( ListSelectionEvent ev ) {
		if ( ev.getValueIsAdjusting()) {
			return;
		}
		int row = table.getSelectedRow();
		for ( int i = 0; i < vecSel.size(); i++ ) {
			SelectionListener l = (SelectionListener )vecSel.get( i );
			l.selectionChanged( row );
		}
	}

	class LegendModel extends AbstractTableModel
	{
		private static final long serialVersionUID = 1L;
		String[] cols = new String[2];
		LegendModel() {
			cols[0] = res.getString( "graph.legend.column1.text" );
			cols[1] = res.getString( "graph.legend.column2.text" );
		}
		public int getColumnCount() {
			return cols.length;
		}
		public String getColumnName( int col ) {
			return cols[col];
		}

		public int getRowCount() {
			if ( listParam == null ) {
				return 0;
			}
			return listParam.size();
		}
		public Object getValueAt( int row, int col ) {
			GraphParam param = (GraphParam )listParam.get( row );
			String ret = null;
			switch ( col ) {
			case 0: ret = ""; break;
			case 1: ret = param.getValue(); break;
			}
			return ret;
		}
	}

	class LegendRenderer extends DefaultTableCellRenderer
	{
		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent( JTable table,
				Object value, boolean isSelected, boolean hasFocus,
				int row, int column )
		{
			GraphParam param = (GraphParam )listParam.get( row );
			setBackground( param.getColor() );
			return this;
		}
	}

	public static interface SelectionListener {
		public void selectionChanged( int row );
	}

}

