package jp.co.sds_corp.hibiya.monitor;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import jp.co.sds_corp.hibiya.connector.ConnectionEvent;
import jp.co.sds_corp.hibiya.connector.DirectConnector;
import jp.co.sds_corp.hibiya.connector.IConnectionListener;
import jp.co.sds_corp.hibiya.connector.IConnector;
import jp.co.sds_corp.hibiya.monitor.checker.ICheckerPanel;
import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.hibiya.server.ICheckerListener;
import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.DOMWriter;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.config.ConfigUtility;
import jp.co.sds_corp.util.daemon.manager.DaemonManager;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.window.ExtFileFilter;
import jp.co.sds_corp.util.window.FontChooser;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* Hibiya Monitorのメインダイアログに表示されるパネルです。
*/
public class MonitorPanel extends JPanel
							implements ICheckerListener, IConnectionListener
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	IConfiguration	monConfig;
	Window			parent;
	SetupParam		setup;
	Vector<IConnector>			vecCon = new Vector<IConnector>();
	Map<String,MonitorParam>	mapPrm = new LinkedHashMap<String,MonitorParam>();
	Vector<CheckerChangeEvent> vecHist = new Vector<CheckerChangeEvent>();
	CmnProperty		prop;

	SetupDialog		setupDlg;
	JSplitPane		sp;
	IConnector		curCon = null;
	IConnector		selCon = null;
	ListTable		curTbl = null;
	HashMap<String,ListTable>	mapLst = new HashMap<String,ListTable>();
	MonitorTree		tree = null;
	JPopupMenu		rMenu;
	JPopupMenu		conMenu;

	ColumnDialog	colDlg;
	CheckerDialog	chkDlg;
	ConnectorDialog	conDlg;
	HistoryDialog	histDlg;
	AboutDialog		about;
	FontChooser		font;

	public MonitorPanel() throws Exception {
		super();
	}

	public void init( IConfiguration config ) throws Exception {
		prop = new CmnProperty( "resource.monitor" );
		parent = (Window )SwingUtilities.getRoot( this );
		monConfig = config.getChild( "Monitor" );
		IConfiguration c;
		c = monConfig.getChild( "Setup" );
		setup = new SetupParam();
		setup.init( c );

		createServer( config );
		createConnectorList( monConfig.getChilds( "Connector" ) );
		createPanelList( monConfig );
		createControls();

		startConnector();
		startServer();
	}

	public void stop() {
		for ( int i = 0; i < vecCon.size(); i++ ) {
			IConnector con = (IConnector )vecCon.get( i );
			con.removeCheckerListener( this );
			con.removeConnectionListener( this );
			con.stop();
		}
	}

	protected void createServer( IConfiguration config )
		throws Exception
	{
		if ( config == null ) {
			return;
		}
		DaemonManager manager = DaemonManager.getInstance();
		manager.init( config );
	}
	protected void startServer() {
		DaemonManager manager = DaemonManager.getInstance();
		manager.start();
	}

	protected void createConnectorList( Iterator<IConfiguration> itr )
		throws Exception
	{
		vecCon.clear();
		IConfiguration c;
		IConnector con;
		while( itr.hasNext() ) {
			c = (IConfiguration )itr.next();
			con = createConnector( c );
			vecCon.add( con );
		}
	}

	protected IConnector createConnector( IConfiguration config )
		throws Exception
	{
		IConnector ret;
		ret = (IConnector )ConfigUtility.createInstance( config );
		ret.setName( config.getAttributes().get( "name" ));
		ret.init( config );
		ret.addCheckerListener( this );
		ret.addConnectionListener( this );

		return ret;
	}

	protected void startConnector() {
		for ( int i = 0; i < vecCon.size(); i++ ) {
			IConnector con = (IConnector )vecCon.get( i );
			con.start();
		}
	}

	protected void createPanelList( IConfiguration config ) throws Exception {
		IConfiguration c;
		mapPrm.clear();
		Iterator<IConfiguration> itr = config.getChilds( "Group" );
		while ( itr.hasNext() ) {
			c = (IConfiguration )itr.next();
			MonitorParam prm = createParam( c );
			if ( mapPrm.get( prm.getName() ) != null ) {
				throw( new Exception( "Group " + prm.getName() + " duplicated." ));
			}

			ICheckerPanel pnl = createPanel( prm );
			mapPrm.put( prm.getName(), prm );
		}
	}

	protected MonitorParam createParam( IConfiguration config )
		throws Exception
	{
		MonitorParam ret = new MonitorParam( setup );
		ret.init( config );
		return ret;
	}

	protected ICheckerPanel createPanel( MonitorParam prm )
		throws Exception
	{
		ICheckerPanel ret = prm.getPanel();
		return ret;
	}

	protected void createControls() throws Exception {
		setLayout( new BorderLayout() );
		Font f = setup.getFont();
		setFont( f );

		curTbl = null;
		createPopup();
		createConnectorPopup();
		createSheet();
		createTree();

		sp = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT );
		JScrollPane sc = new JScrollPane( tree );
		sp.add( sc );

		sp.setOneTouchExpandable( true );
		sp.setDividerLocation( setup.getDivider() );
		add( sp, BorderLayout.CENTER );

		if ( curTbl == null ) {
			JLabel lbl = new JLabel();
			sp.setRightComponent( lbl );
		} else {
			sp.setRightComponent( curTbl.getScrollPane() );
		}

		if ( parent instanceof Frame ) {
			about = new AboutDialog( (Frame )parent );
			chkDlg = new CheckerDialog( (Frame )parent, setup );
			conDlg = new ConnectorDialog( (Frame )parent, setup );
			colDlg = new ColumnDialog( (Frame )parent, setup );
			histDlg = new HistoryDialog( (Frame )parent, setup, vecHist );
			setupDlg = new SetupDialog( (Frame )parent, setup );
		} else {
			about = new AboutDialog( (Dialog )parent );
			chkDlg = new CheckerDialog( (Dialog )parent, setup );
			conDlg = new ConnectorDialog( (Dialog )parent, setup );
			colDlg = new ColumnDialog( (Dialog )parent, setup );
			histDlg = new HistoryDialog( (Frame )parent, setup, vecHist );
			setupDlg = new SetupDialog( (Dialog )parent, setup );
		}
		addCheckerListener( histDlg );
		font = new FontChooser();

		setBounds( setup.getX(), setup.getY(),
					setup.getWidth(), setup.getHeight() );
	}

	protected void createTree() throws Exception {
		tree = new MonitorTree( setup );
		tree.setFont( getFont() );

		tree.addTreeSelectionListener( new TreeSelectionListener() {
			public void valueChanged(TreeSelectionEvent ev) {
				changeSelection( ev );
			}
		});
		tree.addMouseListener( new MouseAdapter() {
			public void mouseReleased(MouseEvent ev) {
				treeClicked( ev );
			}
		});

		for ( int i = 0; i < vecCon.size(); i++ ) {
			IConnector con = (IConnector )vecCon.get( i );
			tree.addConnector( con, mapPrm );
		}
		tree.expandAll();
		addCheckerListener( tree );
	}

	protected void changeSelection( TreeSelectionEvent ev ) {
		DefaultMutableTreeNode node =
			(DefaultMutableTreeNode)tree.getLastSelectedPathComponent();
		if ( node == null ) {
			return;
		}

		NodeInfo info = (NodeInfo )node.getUserObject();
		if ( info == null ) {
			return;
		}
		selCon = info.getConnector();
		if ( info.getParam() == null ) {
			return;
		}
		selectListTable( info.getConnector(), info.getParam() );

		if ( info.getLevel() == NodeInfo.Level.CHECKER ) {
			int nRow = node.getParent().getIndex( node );
			if ( nRow >= 0 && nRow < curTbl.getRowCount() ) {
				curTbl.setRowSelectionInterval( nRow, nRow );
			}
		}
	}

	private void selectListTable( IConnector con, MonitorParam param ) {
		if ( !con.isConnected()) {
			return;
		}
		curCon = con;
		ListTable tbl;
		tbl = findListTable( con, param );
		ListModel model = (ListModel )tbl.getModel();
		model.setConnector( con );
		try {
			model.updateChecker();
			curTbl = tbl;
			int loc = sp.getDividerLocation();
			sp.setRightComponent( curTbl.getScrollPane() );
			sp.setDividerLocation( loc );
		} catch ( Exception ex ) {
			logger.error( "update error", ex );
		}
	}

	void treeClicked( MouseEvent ev ) {
		if ( ev.getClickCount() == 2 ) {
			treeDoubleClicked( ev );
			return;
		}
		if ( SwingUtilities.isRightMouseButton( ev )) {
			showTreePopup( ev );
		}
	}

	void showTreePopup( MouseEvent ev ) {
		NodeInfo info = getNodeInfo( ev.getX(), ev.getY() );
		if ( info == null ) {
			return;
		}
		TreePath path;
		switch ( info.getLevel() ) {
		case CONNECTOR:
			path = tree.getPathForLocation( ev.getX(), ev.getY() );
			tree.setSelectionPath( path );
			if ( conMenu != null ) {
				conMenu.show( tree, ev.getX(), ev.getY() );
			}
			break;
		case GROUP:
			path = tree.getPathForLocation( ev.getX(), ev.getY() );
			tree.setSelectionPath( path );
			showPopup( tree, -1, ev );
			break;
		case CHECKER:
			path = tree.getPathForLocation( ev.getX(), ev.getY() );
			tree.setSelectionPath( path );
			DefaultMutableTreeNode node = getNode( ev.getX(), ev.getY() );
			int nRow = node.getParent().getIndex( node );
			showPopup( tree, nRow, ev );
		default:
			break;
		}
	}

	void treeDoubleClicked( MouseEvent ev ) {
		NodeInfo info = getNodeInfo( ev.getX(), ev.getY() );
		if ( info == null || info.getParam() == null ) {
			return;
		}
		if ( info.getLevel() != NodeInfo.Level.CHECKER ) {
			return;
		}
		IConnector con = info.getConnector();
		if ( !con.isConnected()) {
			return;
		}
		selectListTable( con, info.getParam() );

		setChecker( con, info.getParam(), info.getChecker() );
	}

	NodeInfo nodeInfo = null;
	TreePath nodePath = null;

	protected DefaultMutableTreeNode getNode( int x, int y ) {
		TreePath path = tree.getPathForLocation( x, y );
		if ( path == null ) {
			return null;
		}
		DefaultMutableTreeNode node =
			(DefaultMutableTreeNode)path.getLastPathComponent();
		return node;
	}
	protected NodeInfo getNodeInfo( int x, int y ) {
		DefaultMutableTreeNode node = getNode( x, y );
		if ( node == null ) {
			return null;
		}
		NodeInfo info = (NodeInfo )node.getUserObject();
		return info;
	}

	private ListTable findListTable( IConnector con, MonitorParam prm ) {
		return (ListTable )mapLst.get( prm.getName() );
	}
	private MonitorParam findParam( IChecker mon ) {
		Iterator<String> itr = mapPrm.keySet().iterator();
		while ( itr.hasNext() ) {
			Object key = itr.next();
			MonitorParam prm = (MonitorParam )mapPrm.get( key );
			if ( prm.isValidChecker( mon )) {
				return prm;
			}
		}
		return null;
	}

	protected void createConnectorPopup() {
		JMenuItem	item;
		String	text;
		text = prop.getString( "menu2.item5.text", null );
		if ( text == null ) {
			return;
		}
		conMenu = new JPopupMenu();
		text = prop.getString( "menu2.item5.item1.text" );
		item = new JMenuItem( text, KeyEvent.VK_A );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				addConnector();
			}
		});
		conMenu.add( item );
		text = prop.getString( "menu2.item5.item2.text" );
		item = new JMenuItem( text, KeyEvent.VK_E );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				editConnector();
			}
		});
		conMenu.add( item );
		text = prop.getString( "menu2.item5.item3.text" );
		item = new JMenuItem( text, KeyEvent.VK_R );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				removeConnector();
			}
		});
		conMenu.add( item );
	}

	protected void createSheet() {
		mapLst.clear();
		Iterator<String> itr;
		itr = mapPrm.keySet().iterator();
		while ( itr.hasNext() ) {
			Object key = itr.next();
			MonitorParam prm = (MonitorParam )mapPrm.get( key );

			ListTable lst = new ListTable( setup, prm );
			lst.setFont( getFont() );
			mapLst.put( (String )key, lst );

			ListModel model = (ListModel )lst.getModel();
			addCheckerListener( model );
			lst.addMouseListener2( new MouseAdapter() {
				public void mouseReleased( MouseEvent ev ) {
					sheetClicked( ev );
				}
				public void mouseClicked( MouseEvent ev ) {
					sheetClicked( ev );
				}
			});
		}
	}

	protected void sheetClicked( MouseEvent ev ) {
		if ( SwingUtilities.isRightMouseButton( ev )) {
			if ( ev.getSource() == curTbl.getScrollPane() ) {
				showPopup( curTbl.getScrollPane(), -1, ev );
			} else {
				int nRow = curTbl.rowAtPoint( new Point( ev.getX(), ev.getY() ));
				showPopup( curTbl, nRow, ev );
			}
			return;
		}
		if ( ev.getClickCount() != 2 ) {
			return;
		}
		setChecker();
	}

	protected void createPopup() {
		rMenu = new JPopupMenu();
		JMenuItem	item;
		String	text;
		text = prop.getString( "menu2.item1.text" );
		item = new JMenuItem( text, KeyEvent.VK_S );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				setChecker();
			}
		});
		rMenu.add( item );
		text = prop.getString( "menu2.item2.text" );
		item = new JMenuItem( text, KeyEvent.VK_C );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				setColumn();
			}
		});
		rMenu.add( item );
		rMenu.addSeparator();
		text = prop.getString( "menu2.item3.text" );
		item = new JMenuItem( text, KeyEvent.VK_A );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				addChecker();
			}
		});
		rMenu.add( item );
		text = prop.getString( "menu2.item4.text" );
		item = new JMenuItem( text, KeyEvent.VK_R );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				removeChecker();
			}
		});
		rMenu.add( item );
	}

	/**
	* 右クリックメニューを表示します。
	*/
	void showPopup( Component src, int nRow, MouseEvent ev ) {
		JMenuItem	itemSet = (JMenuItem )rMenu.getComponent( 0 );
		JMenuItem	itemAdd = (JMenuItem )rMenu.getComponent( 3 );
		JMenuItem	itemDel = (JMenuItem )rMenu.getComponent( 4 );
		if ( nRow < 0 || nRow >= curTbl.getRowCount()) {
			nRow = -1;
		} else {
			curTbl.setRowSelectionInterval( nRow, nRow );
		}
		if ( curCon instanceof DirectConnector ) {
			itemSet.setEnabled( true );
			itemAdd.setEnabled( true );
			itemDel.setEnabled( true );
			if ( nRow < 0 ) {
				itemSet.setEnabled( false );
				itemDel.setEnabled( false );
			}
		} else {
			itemAdd.setEnabled( false );
			itemDel.setEnabled( false );
			if ( nRow < 0 ) {
				itemSet.setEnabled( false );
			} else {
				itemSet.setEnabled( true );
			}
		}
		rMenu.show( src, ev.getX(), ev.getY() );
	}

	/**
	* {@link IChecker}を追加します。
	*/
	protected void addChecker() {
		if ( ! ( curCon instanceof DirectConnector )) {
			return;
		}
		MonitorParam prm = curTbl.getParam();
		IChecker chk = null;
		try {
			chk = (IChecker )Class.forName( prm.getChecker() )
						.newInstance();

			setChecker( curCon, prm, chk );
			if ( chkDlg.getResult() == 1 ) {
				curCon.addChecker( chk );
			}
		} catch ( Exception ex ) {
			String title = prop.getString( "monitor.err.title" );
			String msg = prop.getString( "monitor.err1.message" );
			JOptionPane.showMessageDialog( MonitorPanel.this,
				msg + "\n" + ex, title,
				JOptionPane.ERROR_MESSAGE );
			logger.error( msg, ex );
			return;
		}
	}

	/**
	* カラム設定を行います。
	*/
	protected void setColumn() {
		ListModel model = (ListModel)curTbl.getModel();
		MonitorParam prm = curTbl.getParam();
		colDlg.setData( prm );
		WindowUtility.center( this, colDlg );
		colDlg.setVisible( true );
		if ( colDlg.getResult() == 1 ) {
			model.fireTableStructureChanged();
			curTbl.setIconColumn( this );
			model.fireTableDataChanged();
		}
	}

	/**
	* {@link IChecker}を設定します。
	*/
	protected void setChecker() {
		int nRow = curTbl.getSelectedRow();
		if ( nRow < 0 ) {
			return;
		}
		if ( !curCon.isConnected()) {
			return;
		}
		ListModel model = (ListModel)curTbl.getModel();
		IChecker chk = (IChecker )model.getData( nRow );
		MonitorParam prm = curTbl.getParam();
		setChecker( curCon, prm, chk );
	}

	protected void setChecker( IConnector con, MonitorParam prm, IChecker chk )
	{
		chkDlg.setData( con, prm, chk );
		WindowUtility.center( this, chkDlg );
		chkDlg.setVisible( true );
	}

	/**
	* {@link IChecker}を削除します。
	*/
	protected void removeChecker() {
		if ( ! ( curCon instanceof DirectConnector )) {
			return;
		}
		int nRow = curTbl.getSelectedRow();
		if ( nRow < 0 ) {
			return;
		}
		ListModel model = (ListModel)curTbl.getModel();
		IChecker chk = (IChecker )model.getData( nRow );
		MonitorParam prm = curTbl.getParam();
		String title = prop.getString( "monitor.del.title" );
		String msg = prop.getString( "monitor.del.message" )
						+ " " + chk.getType() + " [" + chk.getName() + "]";
		int nRet = JOptionPane.showConfirmDialog( this,
					msg, title, JOptionPane.YES_NO_OPTION );
		if ( nRet != JOptionPane.YES_OPTION ) {
			return;
		}
		try {
			curCon.removeChecker( chk );
		} catch ( Exception ex ) {
			title = prop.getString( "monitor.err.title" );
			msg = prop.getString( "monitor.err2.message" );
			JOptionPane.showMessageDialog( MonitorPanel.this,
				msg + "\n" + ex, title,
				JOptionPane.ERROR_MESSAGE );
			logger.error( msg, ex );
		}
	}

	/**
	* {@link IConnector}を追加します。
	*/
	public void addConnector() {
		WindowUtility.center( this, conDlg );
		conDlg.add();
		if ( conDlg.isOk()) {
			IConnector con = conDlg.getConnector();
			vecCon.add( con );
			con.addCheckerListener( this );
			con.addConnectionListener( this );
			tree.addConnector( con, mapPrm );
			tree.expandAll();
			con.start();
		}
	}
	/**
	* {@link IConnector}を設定します。
	*/
	public void editConnector() {
		if ( selCon == null ) {
			return;
		}
		conDlg.setConnector( selCon );
		WindowUtility.center( this, conDlg );
		conDlg.setVisible( true );
		if ( conDlg.isOk()) {
			selCon.stop();
			selCon.start();
		}
	}
	/**
	* {@link IConnector}を削除します。
	*/
	public void removeConnector() {
		SetupParam.Registry reg = setup.getConnectorType( selCon );
		String title = prop.getString( "connector.del.title" );
		String msg = prop.getString( "connector.del.message" );
		if ( reg == null ) {
			msg += " " + selCon.getClass().getName()
						+ " [" + selCon.getName() + "]";
		} else {
			msg += " " + reg.getType()
						+ " [" + selCon.getName() + "]";
		}
		int nRet = JOptionPane.showConfirmDialog( this,
					msg, title, JOptionPane.YES_NO_OPTION );
		if ( nRet != JOptionPane.YES_OPTION ) {
			return;
		}
		selCon.stop();
		vecCon.remove( selCon );
		tree.removeConnector( selCon );
		selCon = null;
	}

	/**
	* 接続状態に変更があった場合に呼び出されます。
	* @param	ev		接続状態の変更内容を指定します。
	*/
	public void connected( ConnectionEvent ev ) throws Exception {
		IConnector con = (IConnector )ev.getSource();
		Map<String,IChecker> map = con.getCheckerMap();
		Iterator<String> itr = map.keySet().iterator();
		while ( itr.hasNext()) {
			IChecker chk = (IChecker )map.get( itr.next() );
			MonitorParam prm = findParam( chk );
			GraphBufferFactory.addBuffer( con, chk, prm );
		}
		updateConnector( con );
	}

	protected void updateConnector( IConnector con ) {
		try {
			tree.updateConnector( con, mapPrm );
			tree.repaint();
			if ( curTbl == null ) {
				Iterator<String> itr = mapPrm.keySet().iterator();
				while ( itr.hasNext()) {
					MonitorParam prm = (MonitorParam )mapPrm.get( itr.next());
					if ( tree.findGroupNode( con, prm ) != null ) {
						selectListTable( con, prm );
						break;
					}
				}
			}
			if ( curTbl != null ) {
				ListModel model = (ListModel)curTbl.getModel();
				IConnector con2 = model.getConnector();
				if ( con == con2 ) {
					model.updateChecker();
					curTbl.repaint();
				}
			}
		} catch ( Exception ex ) {
			logger.error( "update error", ex );
		}
	}

	/**
	* 接続状態に変更があった場合に呼び出されます。
	* @param	ev		接続状態の変更内容を指定します。
	*/
	public void disconnected( ConnectionEvent ev ) throws Exception {
		repaint();
	}

	/**
	* ICheckerの状態に変更があった場合に呼び出されます。
	* @param	ev		状態の変更内容を指定します。
	*/
	public void checkerChanged( CheckerChangeEvent ev ) throws Exception {
		IChecker chk = (IChecker )ev.getSource();
		if ( ev.getNewStatus() == IChecker.Status.ADD ) {
			addChecker( ev.getConnector(), chk );
			return;
		}
		if ( ev.getNewStatus() == IChecker.Status.DEL ) {
			removeChecker( ev.getConnector(), chk );
			return;
		}
		if ( ev.getNewStatus() != ev.getOldStatus()) {
			vecHist.add( ev );
		}
		fireCheckerChange( ev );
		repaint();
	}

	private void addChecker( IConnector con, IChecker chk ) {
		MonitorParam prm = findParam( chk );
		tree.updateChecker( con, prm );
		ListModel model = (ListModel)curTbl.getModel();
		try {
			model.updateChecker();
		} catch ( Exception ex ) {
			logger.error( "update error", ex );
		}
	}

	private void removeChecker( IConnector con, IChecker chk ) {
		MonitorParam prm = findParam( chk );
		tree.updateChecker( con, prm );
		ListTable tbl = findListTable( con, prm );
		ListModel model = (ListModel)tbl.getModel();
		try {
			model.updateChecker();
		} catch ( Exception ex ) {
			logger.error( "update error", ex );
		}
	}

	Vector<ICheckerListener> vecStat = new Vector<ICheckerListener>();
	/**
	* 状態の変更を受け取るオブジェクトを追加します。
	* @param	l		状態の変更を受け取るオブジェクト
	*/
	public void addCheckerListener( ICheckerListener l ) {
		vecStat.add( l );
	}

	/**
	* 状態の変更を受け取るオブジェクトを削除します。
	* @param	l		状態の変更を受け取るオブジェクト
	*/
	public void removeCheckerListener( ICheckerListener l ) {
		vecStat.remove( l );
	}

	/**
	* 状態変更イベントを通知します。
	* @param	ev		状態変更イベント
	*/
	public void fireCheckerChange( CheckerChangeEvent ev ) {
		for ( int i = 0; i < vecStat.size(); i++ ) {
			ICheckerListener l = (ICheckerListener )vecStat.get( i );
			try {
				l.checkerChanged( ev );
			} catch ( Exception ex ) {
				logger.error( "checkerChanged() error", ex );
			}
		}
	}

	public void clear() {
		tree.removeAll();
		repaint();
	}

	// 新規作成
	public void newConfig() {
		clear();
	}

	// 開く
	public void openConfig() {
		String[] ext = { "xml" };
		String text = prop.getString( "monitor.write.filter" );
		JFileChooser fc = new JFileChooser( new File( "." ).getAbsolutePath() );
		fc.setFileFilter( new ExtFileFilter( text, ext ) );
		int nRet = fc.showOpenDialog( this );
		if ( nRet != JFileChooser.APPROVE_OPTION ) {
			return;
		}
		try {
			File file = fc.getSelectedFile();
			DOMConfiguration config = new DOMConfiguration( file );
			init ( config );
		} catch ( Exception ex ) {
			logger.error( "error", ex );
			String title = prop.getString( "monitor.open.err1.title" );
			String msg = prop.getString( "monitor.open.err1.message" );
			JOptionPane.showMessageDialog( this,
					msg + "\n" + ex.getMessage(), title,
					JOptionPane.ERROR_MESSAGE );
		}
	}

	// 名前をつけて保存
	public void saveConfig() {
		String[] ext = { "xml" };
		JFileChooser fc = new JFileChooser( new File( "." ).getAbsolutePath() ){
			private static final long serialVersionUID = 1L;

			public void approveSelection() {
				File sel = getSelectedFile();
				if ( getFileFilter() instanceof ExtFileFilter ) {
					if ( !sel.getPath().toLowerCase().endsWith( ".xml" )) {
						sel = new File( sel.getPath() + ".xml" );
						setSelectedFile( sel );
					}
				}
				if ( sel.exists() ) {
					String title = prop.getString( "monitor.write.title" );
					String msg = prop.getString( "monitor.write.message" );
					int nRet = JOptionPane.showConfirmDialog( this,
						msg, title,
						JOptionPane.YES_NO_OPTION );
					if ( nRet != JOptionPane.YES_OPTION ) {
						return;
					}
				}
				super.approveSelection();
			}
		};

		String text = prop.getString( "monitor.write.filter" );
		fc.setFileFilter( new ExtFileFilter( text, ext ) );
		int nRet = fc.showSaveDialog( this );
		if ( nRet != JFileChooser.APPROVE_OPTION ) {
			return;
		}

		try {
			writeXml( fc.getSelectedFile() );
		} catch ( Exception ex ) {
			logger.error( "error", ex );
			String title = prop.getString( "monitor.write.err1.title" );
			String msg = prop.getString( "monitor.write.err1.message" );
			JOptionPane.showMessageDialog( this,
					msg + "\n" + ex.getMessage(), title,
					JOptionPane.ERROR_MESSAGE );
		}
	}

	protected void writeXml( File file ) throws Exception {
		DOMConfiguration config = DOMConfiguration.createDocument( "Hibiya" );
		DaemonManager.getInstance().saveConfig( config );

		IConfiguration c;
		IConfiguration mc;
		mc = config.addChild( "Monitor" );

		for ( int i = 0; i < vecCon.size(); i++ ) {
			IConnector con = (IConnector )vecCon.get( i );
			c = mc.addChild( "Connector" );
			con.saveConfig( c );
		}

		c = mc.addChild( "Setup" );
		setup.setBounds( parent.getBounds() );
		setup.setDivider( sp.getDividerLocation() );
		setup.setRowHeight( tree.getRowHeight() );
		setup.saveConfig( c );

		Iterator<String> itr2;
		itr2 = mapLst.keySet().iterator();
		while ( itr2.hasNext()) {
			ListTable tbl = (ListTable )mapLst.get( itr2.next());
			tbl.saveColumn();
		}
		Iterator<String> itr3;
		itr3 = mapPrm.keySet().iterator();
		while ( itr3.hasNext()) {
			MonitorParam prm = (MonitorParam )mapPrm.get( itr3.next());
			c = mc.addChild( "Group" );
			prm.saveConfig( c );
		}

		Writer out = new OutputStreamWriter(
			new FileOutputStream( file ), "UTF-8" );
		DOMWriter writer = new DOMWriter( out );
		writer.write( config, "Hibiya", true );
		// writer.write( config, "Hibiya", null );
		out.flush();
		out.close();

		String title = prop.getString( "monitor.write.end.title" );
		String msg = prop.getString( "monitor.write.end.message" );
		JOptionPane.showMessageDialog( this,
				msg, title,
				JOptionPane.INFORMATION_MESSAGE );
	}

	protected void copyChilds( IConfiguration src, IConfiguration config,
								String key )
	{
		Iterator<IConfiguration> itr;
		itr = src.getChilds( key );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			config.addChild( c );
		}
	}

	public void setOption() {
		WindowUtility.center( this, setupDlg );
		setupDlg.setVisible( true );
	}

	// 履歴表示
	public void showHistory() {
		WindowUtility.center( this, histDlg );
		histDlg.setVisible( true );
	}

	// フォント設定
	public Font setFont() {
		font.setSelectedFont( getFont() );
		Font f = font.showDialog( parent );
		if ( f != null ) {
			setFont( f );
			setup.setFont( f );
			tree.setFont( f );
			Iterator<String> itr = mapLst.keySet().iterator();
			while ( itr.hasNext() ) {
				Object key = itr.next();
				ListTable pnl = (ListTable )mapLst.get( key );
				pnl.setFont( f, this );
			}
			chkDlg.setFont( f );
			colDlg.setFont( f );;
			histDlg.setFont( f );;
			about.setFont( f );;
		}
		return f;
	}

	// Look & Feel設定
	public void setLF( ActionEvent ev ) {
		String cls = ev.getActionCommand();
		try {
			UIManager.setLookAndFeel( cls );

			SwingUtilities.updateComponentTreeUI( parent );
			SwingUtilities.updateComponentTreeUI( setupDlg );
			SwingUtilities.updateComponentTreeUI( chkDlg );
			SwingUtilities.updateComponentTreeUI( histDlg );
			SwingUtilities.updateComponentTreeUI( colDlg );
			SwingUtilities.updateComponentTreeUI( font );
			SwingUtilities.updateComponentTreeUI( about );
			SwingUtilities.updateComponentTreeUI( rMenu );
			SwingUtilities.updateComponentTreeUI( conMenu );

			Iterator<String> itr;
			itr = mapPrm.keySet().iterator();
			while ( itr.hasNext() ) {
				Object key = itr.next();
				MonitorParam prm = (MonitorParam )mapPrm.get( key );
				ICheckerPanel pnl = prm.getPanel();
				SwingUtilities.updateComponentTreeUI((JPanel )pnl );
			}
			itr = mapLst.keySet().iterator();
			while ( itr.hasNext() ) {
				Object key = itr.next();
				ListTable pnl = (ListTable )mapLst.get( key );
				SwingUtilities.updateComponentTreeUI( pnl.getScrollPane() );
			}
		} catch ( Exception ex ) {
			logger.error( "error", ex );
			String title = prop.getString( "monitor.lf.err1.title" );
			String msg = prop.getString( "monitor.lf.err1.message" );
			JOptionPane.showMessageDialog( MonitorPanel.this,
				msg + "\n" + ex, title,
				JOptionPane.ERROR_MESSAGE );
		}
	}

	// ヘルプ
	public void showHelp() {
		WindowUtility.center( this, about );
		about.setVisible( true );
	}

}
